#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package sql;
use strict;

use SrSv::MySQL qw( $dbh );
use SrSv::Text::Format qw( columnar );
use SrSv::IRCd::Event qw( addhandler );
use SrSv::Agent;
use SrSv::Conf2Consts qw( main );
use SrSv::User qw( get_user_nick );
use SrSv::User::Notice;

#use SrSv::Shared qw( %users );
use SrSv::Process::InParent qw( ev_privmsg );

our %users;

our $sqlnick = 'SQLServ';

agent_connect($sqlnick, 'services', 'services.SC.net', '+pqzBGHS', 'Database Query Agent');
agent_join($sqlnick, main_conf_diag);
ircd::setmode($sqlnick, main_conf_diag, '+o', $sqlnick);

addhandler('PRIVMSG', undef, lc $sqlnick, 'sql::ev_privmsg');
sub ev_privmsg {
	my ($src, $dst, $payload) = @_;
	my $user = { NICK => $src, AGENT => $sqlnick };
	#FIXME: More fine grained permissions needed.
	# SELECT is relatively safe. EXPLAIN is too.
	unless(adminserv::is_svsop($user, adminserv::S_ROOT())) {
		notice($user, "Permission denied"); #FIXME: need $err_deny
		return;
	}
	#irssi's splitlong uses ... for beginning and end of a split payload
	$payload =~ s/(^\.\.\.|\.\.\.$)//;
	if($payload =~ /^help/) {
		notice($user, "Sorry, no documentation yet.");
	}
	elsif($payload =~ /^SELECT (.*)$/i) {
		my $statement = $1;
		if ($statement =~ /;$/) {
			SELECT($user, $statement);
		} else {
			#notice($user, "Queuing command\n");
			$users{$src}{STMT} = $statement;
			$users{$src}{CMD} = 'SELECT';
		}
	} elsif($payload =~ /^SHOW CREATE (.*)$/i) {
		my $statement = $1;
		if ($statement =~ /;$/) {
			SHOW_CREATE($user, $statement);
		} else {
			#notice($user, "Queuing command\n");
			$users{$src}{STMT} = $statement;
			$users{$src}{CMD} = 'SHOW CREATE';
		}
	} elsif($payload =~ /^UPDATE (.*)$/i) {
		my $statement = $1;
		if ($statement =~ /;$/) {
			UPDATE($user, $statement);
		} else {
			$users{$src}{STMT} = $statement;
			$users{$src}{CMD} = 'UPDATE';
		}
	} elsif($payload =~ /^INSERT (.*)$/i) {
		my $statement = $1;
		if ($statement =~ /;$/) {
			INSERT($user, $statement);
		} else {
			$users{$src}{STMT} = $statement;
			$users{$src}{CMD} = 'INSERT';
		}
	} elsif($payload =~ /^ALTER (.*)$/i) {
		my $statement = $1;
		if ($statement =~ /;$/) {
			ALTER($user, $statement);
		} else {
			$users{$src}{STMT} = $statement;
			$users{$src}{CMD} = 'ALTER';
		}
	} elsif($payload =~ /^EXPLAIN (.*)$/i) {
		my $statement = $1;
		if ($statement =~ /;$/) {
			EXPLAIN($user, $statement);
		} else {
			$users{$src}{STMT} = $statement;
			$users{$src}{CMD} = 'EXPLAIN';
		}
	} else {
		$users{$src}{STMT} .= ' '.$payload;
		if ($payload =~ /;$/) {
			if(uc($users{$src}{CMD}) eq 'SELECT') {
				SELECT($user, $users{$src}{STMT});
			} elsif(uc($users{$src}{CMD}) eq 'SHOW CREATE') {
				INSERT($user, $users{$src}{STMT});
			} elsif(uc($users{$src}{CMD}) eq 'UPDATE') {
				UPDATE($user, $users{$src}{STMT});
			} elsif(uc($users{$src}{CMD}) eq 'INSERT') {
				INSERT($user, $users{$src}{STMT});
			} elsif(uc($users{$src}{CMD}) eq 'EXPLAIN') {
				UPDATE($user, $users{$src}{STMT});
			} elsif(uc($users{$src}{CMD}) eq 'ALTER') {
				INSERT($user, $users{$src}{STMT});
			} else {
				notice($user, "WTF? '$users{$src}{CMD}' doesn't look like a valid command.");
			}
			delete($users{$src});
		} else {
			#notice($user, "Queuing command\n");
		}
	}
}

sub UPDATE {
	my ($user, $statement) = @_;
	notice($user, "Unsupported command");
}
sub ALTER {
	my ($user, $statement) = @_;
	notice($user, "Unsupported command");
}
sub EXPLAIN {
	my ($user, $statement) = @_;
	notice($user, "Unsupported command");
}
sub INSERT {
	my ($user, $statement) = @_;
	notice($user, "Unsupported command");
}

sub SELECT {
	my ($user, $statement) = @_;
	my $arrayRef;
	my @names;
	eval {
		my $sth = $dbh->prepare("SELECT $statement");
		$sth->execute();
		@names = @{$sth->FETCH('NAME')};
		$arrayRef = $sth->fetchall_arrayref();
	};
	if($@) {
		ircd::debug("AIEEEEE! $@");
		notice($user, "AIEEEEE!",  $@, "SELECT $statement");
	} elsif(scalar(@$arrayRef)) {
		notice($user, columnar( { BORDER => 1, NOHIGHLIGHT => 1 }, [@names], @$arrayRef ) );
	} else {
		notice($user, "Empty result.");
	}
}

sub SHOW_CREATE {
	my ($user, $statement) = @_;
	my $data;
	eval {
		my $sth = $dbh->prepare("SHOW CREATE $statement");
		$sth->execute();
		(undef, $data) = $sth->fetchrow_array();
	};
	if($@) {
		ircd::debug("AIEEEEE! $@");
		notice($user, "AIEEEEE!",  $@, "SHOW CREATE $statement");
	} elsif($data) {
		my @rows = map { [$_] } split("\n", $data);
		notice($user, columnar( { BORDER => 1, NOHIGHLIGHT => 1 }, @rows ) );
	} else {
		notice($user, "Empty result.");
	}
}


sub init { }
sub begin { }
sub end { }
sub unload { }

1;
